import sqlite3
from config import DB_FILE_PATH
from logging_script import logger

conn = sqlite3.connect(DB_FILE_PATH, check_same_thread=False)
c = conn.cursor()


import sqlite3
from config import DB_FILE_PATH

conn = sqlite3.connect(DB_FILE_PATH, check_same_thread=False)
c = conn.cursor()

def database_dependencies():
    c.execute('''
        CREATE TABLE IF NOT EXISTS users (
            user_id INTEGER PRIMARY KEY
        )
    ''')
    c.execute('''
    CREATE TABLE IF NOT EXISTS ads (
        ad_id TEXT PRIMARY KEY,
        category TEXT,
        created_at TIMESTAMP DEFAULT CURRENT_TIMESTAMP
    )
''')
    c.execute('''
        CREATE TABLE IF NOT EXISTS user_ads (
            user_id INTEGER,
            ad_id TEXT,
            PRIMARY KEY (user_id, ad_id),
            FOREIGN KEY (user_id) REFERENCES users (user_id),
            FOREIGN KEY (ad_id) REFERENCES ads (ad_id)
        )
    ''')
    c.execute('''
        CREATE TABLE IF NOT EXISTS permissions (
            user_id INTEGER PRIMARY KEY,
            perm1 BOOLEAN DEFAULT 0,
            perm2 BOOLEAN DEFAULT 0,
            perm3 BOOLEAN DEFAULT 0,
            perm4 BOOLEAN DEFAULT 0,
            count INTEGER DEFAULT 0,
            FOREIGN KEY (user_id) REFERENCES users (user_id)
        )
    ''')

    c.execute('''
    CREATE TABLE IF NOT EXISTS ads_detail (
        id TEXT PRIMARY KEY ,
        title TEXT,
        address TEXT,
        link TEXT,
        price TEXT,
        description TEXT,
        phone TEXT,
        photo_link TEXT,
        date TEXT,
        platform TEXT,
        FOREIGN KEY (id) REFERENCES ads(ad_id) ON DELETE SET NULL
        )
    ''')
    conn.commit()

# Function to load user_ids from database
def load_user_ids():
    c.execute('SELECT user_id FROM users')
    return [row[0] for row in c.fetchall()]

# Function to save a user_id to the database
def save_user_id(user_id):
    c.execute('INSERT OR IGNORE INTO users (user_id) VALUES (?)', (user_id,))
    conn.commit()

# Function to save ad ID to the database
def save_ad_id(ad_id, category):
    c.execute('INSERT OR IGNORE INTO ads (ad_id, category) VALUES (?, ?)', (ad_id,category))
    conn.commit()

def ad_id_exists(ad_id):
    c.execute('SELECT 1 FROM ads WHERE ad_id = ?', (ad_id,))
    return c.fetchone() is not None

# Function to save which ad was sent to which user
def save_user_ad(user_id, ad_id):
    c.execute('INSERT OR IGNORE INTO user_ads (user_id, ad_id) VALUES (?, ?)', (user_id, ad_id))
    conn.commit()

# Function to check if an ad was already sent to a user
def ad_already_sent(user_id, ad_id):
    c.execute('SELECT 1 FROM user_ads WHERE user_id = ? AND ad_id = ?', (user_id, ad_id))
    return c.fetchone() is not None

# Function to check if a user has a specific permission
def has_permission(user_id):
    c.execute(f'SELECT 1 FROM permissions WHERE user_id = ?', (user_id,))
    result = c.fetchone()
    return result is not None

# Function to revoke all permissions by deleting the user from all tables
def revoke_all_permissions(user_id):
    # Delete the user from the permissions table
    c.execute('DELETE FROM permissions WHERE user_id = ?', (user_id,))
    # Delete the user from the users table
    c.execute('DELETE FROM users WHERE user_id = ?', (user_id,))
    # Delete the user from the user_ads table
    c.execute('DELETE FROM user_ads WHERE user_id = ?', (user_id,))
    conn.commit()

# Function to get all users
def get_all():
    c.execute(f'SELECT user_id FROM users')
    return [row[0] for row in c.fetchall()]


# Function to set user's permissions
def set_user_permissions(user_id, perm1, perm2, perm3, perm4, count):
    # Insert the user into the users table if they do not already exist
    save_user_id(user_id)
    # Insert or update the user's permissions
    c.execute('''
        INSERT INTO permissions (user_id, perm1, perm2, perm3, perm4, count) 
        VALUES (?, ?, ?, ?, ?, ?)
        ON CONFLICT(user_id) 
        DO UPDATE SET perm1 = excluded.perm1, perm2 = excluded.perm2, perm3 = excluded.perm3, perm4 = excluded.perm4, count=excluded.count
    ''', (user_id, perm1, perm2, perm3, perm4, count))
    conn.commit()


def get_user_details(user_id):
    c.execute('''
        SELECT users.user_id, permissions.perm1, permissions.perm2, permissions.perm3, permissions.perm4, permissions.count
        FROM users
        LEFT JOIN permissions ON users.user_id = permissions.user_id
        WHERE users.user_id = ?
    ''', (user_id,))
    return c.fetchone()

# To get the last 6 ads in a given category
def get_5_ads_by_platform(category, user_id):
    # Get 4 ads where platform is 'divar' and not in user_ads for the given user_id
    c.execute('''
        SELECT ads.ad_id 
        FROM ads
        JOIN ads_detail ON ads.ad_id = ads_detail.id
        LEFT JOIN user_ads ON ads.ad_id = user_ads.ad_id AND user_ads.user_id = ?
        WHERE ads.category = ? AND ads_detail.platform = ? AND user_ads.ad_id IS NULL
        ORDER BY ads.created_at DESC
        LIMIT 4
    ''', (user_id, category, '#دیوار'))
    divar_ads = [row[0] for row in c.fetchall()]

    # Get 2 ads where platform is not 'divar' and not in user_ads for the given user_id
    c.execute('''
        SELECT ads.ad_id 
        FROM ads
        JOIN ads_detail ON ads.ad_id = ads_detail.id
        LEFT JOIN user_ads ON ads.ad_id = user_ads.ad_id AND user_ads.user_id = ?
        WHERE ads.category = ? AND ads_detail.platform != ? AND user_ads.ad_id IS NULL
        ORDER BY ads.created_at DESC
        LIMIT 2
    ''', (user_id, category, '#دیوار'))
    non_divar_ads = [row[0] for row in c.fetchall()]

    return divar_ads + non_divar_ads


def get_last_five_ads_not_in_user_ads(category, user_id):
    c.execute('''
        SELECT ads_detail.*
        FROM ads_detail
        JOIN ads ON ads_detail.id = ads.ad_id
        LEFT JOIN user_ads ON ads_detail.id = user_ads.ad_id AND user_ads.user_id = ?
        WHERE user_ads.ad_id IS NULL AND ads.category = ?
        ORDER BY ads_detail.date DESC
        LIMIT 5
    ''', (user_id, category))
    return c.fetchall()


def user_ad_exists(user_id, ad_id):
    c.execute('SELECT 1 FROM user_ads WHERE user_id = ? AND ad_id = ?', (user_id, ad_id))
    return c.fetchone() is not None

def get_count(user_id):
    c.execute('SELECT count FROM permissions WHERE user_id = ?', (user_id,))
    row = c.fetchone()
    if row:
        return row[0]
    else:
        return None

def increment_count(user_id, count):
    current_count = get_count(user_id)
    
    if current_count is not None:
        new_count = current_count + count
        c.execute('UPDATE permissions SET count = ? WHERE user_id = ?', (new_count, user_id))
    
    conn.commit()
    return new_count


def save_ad_and_photos(_id, title, address, link, price, description, phone, photo_link, date, platform):
    logger.info(f" {title} ")
    # Insert a new ad into the ads_detail table
    c.execute('''
        INSERT INTO ads_detail (id, title, address, link, price, description, phone, photo_link, date, platform)
        VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?)
    ''', (_id, title, address, link, price, description, phone, photo_link, date, platform))
    
    conn.commit()


def get_ad_details(ad_id):
    c.execute('''
        SELECT id, title, address, link, price, description, phone, photo_link, date, platform
        FROM ads_detail
        WHERE id = ?
    ''', (ad_id,))
    return c.fetchone()